CREATE DATABASE IF NOT EXISTS elastix_address_book
DEFAULT CHARACTER SET utf8;

GRANT SELECT, INSERT, UPDATE, DELETE ON `elastix_address_book`.* TO 'asteriskuser'@'localhost';

USE elastix_address_book;

CREATE TABLE IF NOT EXISTS contact_directory
(
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */
    directory_name  VARCHAR(64) NOT NULL,
    directory_desc  TEXT    NOT NULL,

    /* ID en acl.db del usuario ACL de Elastix que es responsable de este
     * directorio. Si es NULL, se asume directorio global. */
    iduser  INT UNSIGNED,

    PRIMARY KEY (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO contact_directory VALUES
    (NULL, 'internal', 'Contacts matching PBX extensions', NULL),
    (NULL, 'external', 'External contacts by users', NULL);

CREATE TABLE IF NOT EXISTS contact (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */

    /* ID en acl.db del usuario ACL de Elastix que es responsable de este contacto */
    iduser  INT UNSIGNED,

    /* Bandera que indica si el contacto es público (1) o privado (0) - reemplaza a status=isPublic/isPrivate */
    visibility  BOOLEAN NOT NULL DEFAULT 0,

    PRIMARY KEY (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS contact_directory_membership
(
    id_directory    INT UNSIGNED NOT NULL,
    id_contact      INT UNSIGNED NOT NULL,

    FOREIGN KEY (id_directory) REFERENCES contact_directory (id),
    FOREIGN KEY (id_contact) REFERENCES contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Números telefónicos asociados al contacto */
CREATE TABLE IF NOT EXISTS contact_phone_number (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */
    id_contact      INT UNSIGNED NOT NULL,                  /* relación al contacto */
    phone_number    VARCHAR(64) NOT NULL,                   /* el número en cuestión */
    extension       VARCHAR(32),                            /* extensión opcional a marcar luego de teléfono */

    /* rol asociado para el teléfono */
    phone_role      ENUM('work', 'home', 'mobile', 'fax') NOT NULL DEFAULT 'work',

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Correos asociados al contacto */
CREATE TABLE IF NOT EXISTS contact_email (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */
    id_contact      INT UNSIGNED NOT NULL,                  /* relación al contacto */

    email           VARCHAR(128) NOT NULL,

    /* rol asociado para el correo */
    email_role      ENUM('work', 'personal') NOT NULL DEFAULT 'personal',

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Identidades de mensajería instantánea asociados al contacto */
CREATE TABLE IF NOT EXISTS contact_im_address (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */
    id_contact      INT UNSIGNED NOT NULL,                  /* relación al contacto */

    im_address      VARCHAR(128) NOT NULL,

    /* rol asociado para la cuenta de IM */
    im_role      ENUM('work', 'personal') NOT NULL DEFAULT 'personal',

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Atributos adicionales asociados al contacto */
CREATE TABLE IF NOT EXISTS contact_attribute (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */
    id_contact      INT UNSIGNED NOT NULL,                  /* relación al contacto */

    attribute_key   VARCHAR(32) NOT NULL,
    attribute_value TEXT NOT NULL,

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/*
Atributos para compatibilidad con address_book.db:
name: nombre del contacto
last_name: apellidos del contacto

picture: nombre base del archivo de imagen en /var/www/address_book_images/

notes: notas variadas sobre el contacto

address: dirección del contacto
city: ciudad de la dirección
province: provincia/estado/departamento de dirección
(OJO: no hay país)

company: compañía del contacto
department: departamento dentro de compañía para contacto
company_contact: nombre del contacto responsable dentro de la compañía
contact_rol: cargo ejercido por contacto dentro de compañía
*/

/* Objetivos conocidos de sincronización */
CREATE TABLE IF NOT EXISTS contact_sync_target (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,
    target_name     VARCHAR(64) NOT NULL,
    target_desc     TEXT NOT NULL,

    PRIMARY KEY (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO contact_sync_target VALUES
    (NULL, 'mailchimp', 'MailChimp Mass Mailer'),
    (NULL, 'google', 'Google Contacts');

/* Configuración por usuario para cada objetivo conocido */
CREATE TABLE IF NOT EXISTS contact_sync_user_config (
    id              INT UNSIGNED NOT NULL AUTO_INCREMENT,   /* clave primaria */
    id_synctarget   INT UNSIGNED NOT NULL,

    /* ID en acl.db del usuario ACL de Elastix */
    iduser          INT UNSIGNED NOT NULL,

    attribute_key   VARCHAR(64) NOT NULL,
    attribute_value TEXT NOT NULL,

    PRIMARY KEY (id),
    UNIQUE KEY (id_synctarget, iduser, attribute_key),
    FOREIGN KEY (id_synctarget) REFERENCES contact_sync_target(id)
);

/* Configuración de objetivo de sincronización para cada directorio */
CREATE TABLE IF NOT EXISTS contact_sync_directory_config (
    id_directory    INT UNSIGNED NOT NULL,
    id_synctarget   INT UNSIGNED NOT NULL,

    /* ID en acl.db del usuario ACL de Elastix. Esto se requiere para que un
     * usuario pueda privadamente agregar configuración a un directorio público. */
    iduser          INT UNSIGNED NOT NULL,

    attribute_key   VARCHAR(64) NOT NULL,
    attribute_value TEXT NOT NULL,

    FOREIGN KEY (id_directory) REFERENCES contact_directory (id),
    FOREIGN KEY (id_synctarget) REFERENCES contact_sync_target(id)
);



